package jitas.core;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.rmi.RemoteException;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import jess.JessException;

import jitas.exception.*;

/**
 * The Admin class is the doorway to JITAS for the Administration GUI
 * @author	DeathMarch (c314g2)
 * @version	08 October 2007
 */
public class Admin {
	
	private Tutor tutor;
	
	/**
	 * The default constructor for the admin class.
	 * This method should be called whenever an application wants to run the admin tool.
	 * All subsequent method calls should be made through the initial object.
	 * @throws IOException 
	 */
	public Admin() throws IOException {
		tutor = new Tutor();
		for(String domainName : getAllDomainNames()) {
			try {
				tutor.loadDomain(domainName);
			} catch (DomainNotFoundException e) {}
		}
	}
	
	/**
	 * Constructor that takes in a file path for the domains.xml file
	 * @param domainsFilePath The path to an xml file containing information
	 * 	about all the domains. The path must include the filename.
	 * @throws IOException 
	 */
	public Admin (String domainsFilePath) throws IOException {
		tutor = new Tutor(domainsFilePath);
		for(String domainName : getAllDomainNames()) {
			try {
				tutor.loadDomain(domainName);
			} catch (DomainNotFoundException e) {}
		}
	}
	
	/**
	 * Constructor which takes in a current Tutor object, to use that
	 * tutor's domainManager, and set of Students.
	 * @param tutor The Tutor object which is currently being used.
	 * @throws IOException
	 */
	public Admin (Tutor tutor) throws IOException {
		this.tutor = tutor;
		for(String domainName : getAllDomainNames()) {
			try {
				tutor.loadDomain(domainName);
			} catch (DomainNotFoundException e) {}
		}
	}
	
	/**
	 * Returns all of the students usernames which have been
	 * read in from xml.
	 * @return usernames A Sorted Set of all the students usernames.
	 */
	public SortedSet<String> getStudentUsernames() {
		return new TreeSet<String>(tutor.getAllStudentUsernames());
	}
	
	/**
	 * Returns a Sorted Set of all known domain names
	 * 
	 * @return A Sorted Set of all the known domain names
	 */
	public SortedSet<String> getAllDomainNames() {
		SortedSet<String> domains = null;
		try {
			domains = new TreeSet<String>(tutor.getAllDomainNames());
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		return domains;
	} 
	
	/**
	 * Tells whether a student has a student model within a certain domain
	 * @param userName - the students username
	 * @param domainName - the domain name we wish to check
	 * @return true if the student has a student model in the domain, else false
	 * @throws UserNotFoundException
	 */
	public boolean studentHasAccountInDomain(String userName, String domainName) 
	throws UserNotFoundException {
		return tutor.isStudentAllowedInDomain(userName, domainName);
	}
	
	
	/**
	 * Tells whether a domain is a Non-Restricted or Restricted
	 * @param domainName
	 * @return True if the domain is not Restricted, false if the domain is Restricted
	 * @throws DomainNotFoundException
	 * @throws DomainNotLoadedException 
	 * @throws JessException
	 */
	public boolean userCreationAllowed(String domainName) throws DomainNotFoundException, DomainNotLoadedException {
		return tutor.isUserCreationAllowed(domainName);
	}
	
	/**
	 * gets the login status of a student for a particular domain
	 * @param userName The students domain name
	 * @param domainName The domain we wish to check
	 * @return True if the student is logged in else false
	 * @throws RemoteException 
	 * @throws UserNotFoundException If the user does not exist
	 */
	public boolean getLoginStatus(String userName, String domainName) throws UserNotFoundException, RemoteException {
		return tutor.getLoginStatus(userName, domainName);
	}
	
	/**
	 * Logs the Student out of the specified domain
	 * @param userName - the students username
	 * @param domainName - the domain the student wishes to log out of
	 * @throws RemoteException 
	 * @throws UserNotLoggedInException 
	 * @throws UserNotFoundException 
	 * @throws DomainNotLoadedException 
	 * @throws DomainNotFoundException 
	 */
	public void logout(String userName, String domainName) 
	throws DomainNotFoundException, DomainNotLoadedException, UserNotFoundException, 
	UserNotLoggedInException, RemoteException {
		tutor.logout(userName, domainName);
	}
	
	/**
	 * Adds a new student to JITAS.  The Set of domainNames is all of the domains 
	 * that the new student can log into
	 * @param userName Student's user name
	 * @param password Student's password
	 * @param domainNames All the domains the student can log into
	 * @throws RemoteException 
	 * @throws StudentModelAlreadyExistsException 
	 * @throws IllegalUserNameException 
	 * @throws DomainNotLoadedException 
	 * @throws DomainNotFoundException 
	 * @throws AddUserDeniedException 
	 * @throws UserNotFoundException 
	 */
	public void addUser(String userName, String password, Set<String> domainNames) 
	throws AddUserDeniedException, DomainNotFoundException, DomainNotLoadedException, 
	IllegalUserNameException, StudentModelAlreadyExistsException, RemoteException {
		boolean first = true;
		for(String domainName : domainNames) {
			tutor.loadDomain(domainName);
			if(first) {
				tutor.addUser(userName, password, domainName, true);
				first = false;
			} else {
				try {
					tutor.addAllowedDomain(userName, domainName, true);
				} catch (UserNotFoundException e) {
					//this wont happen because we just added the user
				}
			}
		}
	}
	
	/**
	 * Removes the students student model from the specified domain.
	 * Does not delete the student altogether - just removes the student information
	 * from the specified domain
	 * @param userName
	 * @param domainName
	 * @throws RemoteException 
	 * @throws UserNotLoggedInException 
	 * @throws UserNotFoundException 
	 * @throws DomainNotLoadedException 
	 * @throws DomainNotFoundException 
	 */
	public void removeUser(String userName, String domainName) 
	throws DomainNotFoundException, DomainNotLoadedException, UserNotFoundException, 
	UserNotLoggedInException, RemoteException {
		tutor.removeUser(userName, domainName);
	}
	
	/**
	 * Deletes the student from the entire JITAS System, including
	 * all domain specific information about that student
	 * @param userName Student's user name
	 * @throws RemoteException 
	 * @throws UserNotLoggedInException 
	 * @throws UserNotFoundException 
	 * @throws DomainNotLoadedException 
	 * @throws DomainNotFoundException 
	 */
	public void deleteUser(String userName) 
	throws DomainNotFoundException, DomainNotLoadedException, UserNotFoundException, 
	UserNotLoggedInException, RemoteException {
		tutor.deleteUser(userName);
	}
	
	/**
	 * Gives a student access to a specific domain
	 * @param userName The student's username
	 * @param domainName The domain the student will have access to
	 * @throws RemoteException 
	 * @throws UserNotFoundException 
	 * @throws StudentModelAlreadyExistsException 
	 * @throws DomainNotLoadedException 
	 * @throws DomainNotFoundException 
	 * @throws AddUserDeniedException 
	 */
	public void addAllowedDomain(String userName, String domainName) 
	throws AddUserDeniedException, DomainNotFoundException, DomainNotLoadedException, 
	StudentModelAlreadyExistsException, UserNotFoundException, RemoteException {
		tutor.addAllowedDomain(userName, domainName);
	}

	/**
	 * gets a set of all the students usernames in JITAS
	 * @return a set of all of the students usernames in JITAS
	 */
	public Set<String> getAllStudentUsernameStrings() {	
		return new TreeSet<String>(tutor.getAllStudentUsernames());
	}
	
	/**
	 * Gets all the students usernames for a specific domain
	 * @param currentFilter The domain the students will belong to
	 * @return set A set of all the students usernames in the filtered domain
	 */
	public Set<String> getStudentUsernameStrings(String currentFilter) {
		return new TreeSet<String>(tutor.getAllStudentUsernames(currentFilter));
	}
	
	/**
	 * loads all of the students usernames from xml into the stored set
	 * of students usernames in this class.
	 * @throws FileNotFoundException 
	 * @throws FileNotFoundException
	 */
	public void refreshStudentList() throws FileNotFoundException {
		tutor.refreshStudentList();
	}	
}
