package jitas.core;

import jitas.exception.*;

import java.rmi.RemoteException;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import jess.JessException;

/**
 * The interface class for the Java Intelligent Tutor Authoring Shell (JITAS)
 * class library (core classes). All other applications will use this class
 * to interface with JITAS. This class provides all the necessary functionality
 * for a higher level application to act as an Intelligent Tutoring System.
 * 
 * @author	DeathMarch (c314g2) - David Thomson, Farah Mohamad, Louis Sayers, Lu Pan, Robert Lechte, Yuganeethen Yugaraja
 * @version	24 August 2007
 */
public interface TutorInterface {

	/**
	 * Login method. This must be called on each tutor for each user before
	 * they can do anything else. A user can log into domains with their global 
	 * login details. If the user does not have permissions for a particular domain,
	 * or if the domain does not exist, then an InvalidLoginException is thrown.
	 * If for some reason the student doesnt have a StudentModel for the domain,
	 * but has already been added to it (eg their Model got deleted from the XML file, 
	 * then this method will create a new fresh StudentModel for that user.
	 * If this method does not throw any exceptions then it was succesfull.
	 * @param userName The user name of the user.
	 * @param password The password for the user.
	 * @param domainName The Domain name the user wants to log into.
	 * @throws DomainNotFoundException If the domain is not found.
	 * @throws DomainNotLoadedException If the domain is not loaded.
	 * @throws InvalidLoginException If the login is invalid (wrong password).
	 * @throws UserNotFoundException If the user does not exist.
	 * @throws RemoteException 
	 */
	public void login(String userName, String password, String domainName)
	throws DomainNotFoundException, DomainNotLoadedException, InvalidLoginException,
	UserNotFoundException, RemoteException;
	
	/**
	 * Logs a specified user out of the domain.
	 * If this method does not throw any exceptions it was succesfull.
	 * @param userName The user to be logged out.
	 * @param domainName The Domain name the user wants to log out of.
	 * @throws DomainNotFoundException If the domain is not found.
	 * @throws DomainNotLoadedException If the domain is not loaded.
	 * @throws UserNotFoundException If the user could not be found.
	 * @throws UserNotLoggedInException If the user was not logged in.
	 * @throws RemoteException 
	 */
	public void logout(String userName, String domainName)
	throws DomainNotFoundException, DomainNotLoadedException, UserNotFoundException,
	UserNotLoggedInException, RemoteException;
	
	/**
	 * Get the login status for the user, for all the domains a student is logged into
	 * (i.e. if they are logged into the domain or not)
	 * @param userName The user name of the user who wants to log out.
	 * @param domainName The Domain name.
	 * @return true if they are logged in, false otherwise.
	 * @throws UserNotFoundException If the user could not be found.
	 * @throws RemoteException 
	 */
	public boolean getLoginStatus(String userName, String domainName)
	throws UserNotFoundException, RemoteException;
	
	/**
	 * Adds a user to the JITAS system and authorize them to access domains. 
	 * If the user already exists, then this method will attempt to give the user 
	 * access to the specific domain. If the domain does not allow users to add themselves
	 * then a AddUserDeniedException will be thrown. If the user does not already have an 
	 * account (they are new), then all necessary account data will be created.
	 * This method does NOT automatically log the user in; the login method must be 
	 * explicitly called after the user has been added.
	 * If this method does not throw any exceptions then it was succesfull.
	 * @param userName The user name of the new user.
	 * @param password The password for the new user.
	 * @param domainName The Domain name the user wants to create an account for.
	 * @throws AddUserDeniedException If new users are not allowed to be added to the domain.
	 * @throws DomainNotFoundException If the domain could not be found.
	 * @throws DomainNotLoadedException If the requested domain is not loaded.
	 * @throws IllegalUserNameException If the user specified an illegal username.
	 * @throws StudentModelAlreadyExistsException If a student model already exists.
	 * @throws RemoteException 
	 */
	public void addUser(String userName, String password, String domainName)
	throws AddUserDeniedException, DomainNotFoundException, DomainNotLoadedException,
	IllegalUserNameException, StudentModelAlreadyExistsException, RemoteException;
	
	/**
	 * Remove a user from a domain. The user must be logged in to run this method.
	 * This method will delete the user history from the domain xml files.
	 * This method however will not Delete a user, and they will still be able to log
	 * into any other domains of which they have access, and accounts for.
	 * @param userName The user name of the user.
	 * @param domainName The name of the Domain the user is to be removed from.
	 * @throws DomainNotFoundException If the domain could not be found.
	 * @throws DomainNotLoadedException If the requested domain is not loaded.
	 * @throws UserNotFoundException If the user could not be found.
	 * @throws UserNotLoggedInException If the user was not logged in.
	 * @throws RemoteException
	 */
	public void removeUser(String userName, String domainName)
	throws DomainNotFoundException, DomainNotLoadedException, UserNotFoundException,
	UserNotLoggedInException, RemoteException;
	
	/**
	 * Delete a user from the JITAS system. The user must be logged in to run this method.
	 * This method will delete the user history from the domain xml files and the JITAS
	 * system (students.xml). This method will Delete a user, and they will not be able 
	 * to log into any domains.
	 * @param userName The user name of the user.
	 * @throws DomainNotFoundException If the domain could not be found.
	 * @throws DomainNotLoadedException If the user is logged into a domain that's not loaded.
	 * @throws UserNotFoundException If the user could not be found.
	 * @throws UserNotLoggedInException If the user was not logged in.
	 * @throws RemoteException
	 */
	public void deleteUser(String userName)
	throws DomainNotFoundException, DomainNotLoadedException, UserNotFoundException,
	UserNotLoggedInException, RemoteException;
	
	/**
	 * This method will attempt to give the user access to the specific domain. 
	 * If the domain does not allow users to add themselves then a AddUserDeniedException 
	 * will be thrown. Allows a student to log into a new domain.
	 * @param userName the user name of the user
	 * @param domainName the name of the domain the student wishes to have access to.
	 * @throws AddUserDeniedException If new users are not allowed to be added to the domain.
	 * @throws DomainNotFoundException thrown if the domain with domainName is not found.
	 * @throws DomainNotLoadedException If the requested domain is not loaded.
	 * @throws StudentModelAlreadyExistsException If a student model already exists
	 * @throws UserNotFoundException thrown if the student with user name userName is not found
	 * @throws RemoteException
	 */
	public void addAllowedDomain(String userName, String domainName)
	throws AddUserDeniedException, DomainNotFoundException, DomainNotLoadedException,
	StudentModelAlreadyExistsException, UserNotFoundException, RemoteException;
	
	/**
	 * Change user's existing password to a new password. The user must be logged in to 
	 * run this method.
	 * @param userName The user name of the user.
	 * @param oldPassword The users current password
	 * @param newPassword The new password that the user wants to change to.
	 * @throws InvalidPasswordException if the current password given is invalid
	 * @throws UserNotLoggedInException If the user was not logged in.
	 * @throws RemoteException
	 */
	public void changePassword(String userName, String oldPassword, String newPassword)
	throws InvalidPasswordException, UserNotFoundException, RemoteException;
	
	/**
	 * Returns a set of names of all the Domains that JITAS "knows" about. 
	 * This can be called before a user logs in.
	 * @return A set of domain names
	 * @throws RemoteException
	 */
	public Set<String> getAllDomainNames()
	throws RemoteException;
	
	/**
	 * Gets all the names for the currently loaded domains as a map
	 * of <name, longName> pairs. 
	 * Note: there is no garantee that any particular long name is not null.
	 * @return A map of name, longName pairs
	 * @throws RemoteException
	 */
	public Map<String, String> getAllLongDomainNames()
	throws RemoteException;
	
	/**
	 * Returns a set of all the subdomain names in a given domain, as Strings.
	 * A domain must be loaded before this method can be called successfully.
	 * @param domainName The name of the domain.
	 * @return A set of subdomain names.
	 * @throws DomainNotFoundException if the domain is not found (or not loaded).
	 * @throws DomainNotLoadedException If the requested domain is not loaded
	 * @throws RemoteException 
	 */
	public Set<String> getSubdomains(String domainName)
	throws DomainNotFoundException, DomainNotLoadedException, RemoteException;
	
	/**
	 * Loads a specified Domain from a .xml file. This includes loading all the problems and
	 * constraints. This can safely be called on a already loaded domain.
	 * @param domainName The name of the domain you want to load.
	 * @throws DomainNotFoundException if the Domain is not found.
	 * @throws JessException If something goes wrong in Jess (the rule engine)
	 * @throws RemoteException 
	 */
	public void loadDomain(String domainName)
	throws DomainNotFoundException, JessException, RemoteException;
	
	/**
	 * Allows a user to select a subdomain to work on.
	 * @param userName The name of the user.
	 * @param domainName The name of the domain the student is working on.
	 * @param subdomainName The name of the subdomain the student wants to work on.
	 * @throws DomainNotFoundException If the domain could not be found.
	 * @throws DomainNotLoadedException If the requested domain is not loaded.
	 * @throws ProblemsNotFoundException If no problems could be found for the subdomain.
	 * @throws SubdomainNotFoundException If the Subdomain is not found.
	 * @throws UserNotFoundException If the user could not be found.
	 * @throws UserNotLoggedInException If the user is not logged in to the domain.
	 * @throws RemoteException 
	 */
	public void selectSubdomain(String userName, String subdomainName, String domainName)
	throws DomainNotFoundException, DomainNotLoadedException, ProblemsNotFoundException,
	SubdomainNotFoundException, UserNotFoundException, UserNotLoggedInException,
	RemoteException;
	
	/**
	 * Gets the next problem, specified by the user.
	 * @param problemNumber The number(id) of the problem the user wants.
	 * @param userName The name of the user.
	 * @param domainName The name of the domain.
	 * @return The problem object
	 * @throws DomainNotFoundException If the domain cannot be found.
	 * @throws DomainNotLoadedException If the requested domain is not loaded.
	 * @throws InvalidProblemException If the problem cannot be found.
	 * @throws UserNotFoundException If the user could not be found.
	 * @throws UserNotLoggedInException If the user is not logged in to the domain.
	 * @throws RemoteException 
	 */
	public Problem getNextProblem(int problemNumber, String userName, String domainName)
	throws DomainNotFoundException, DomainNotLoadedException, InvalidProblemException,
	UserNotFoundException, UserNotLoggedInException, RemoteException;
	
	/**
	 * Gets the next problem, based on the current problem number.
	 * The problem with the next highest id number that has not been completed
	 * by the student is returned.
	 * @param userName The name of the user who wants the problem.
	 * @param domainName The name of the domain.
	 * @return The problem object
	 * @throws DomainNotFoundException If the domain cannot be found.
	 * @throws DomainNotLoadedException If the requested domain is not loaded.
	 * @throws InvalidProblemException  If the problem cannot be found.
	 * @throws UserNotFoundException If the user could not be found.
	 * @throws UserNotLoggedInException If the user is not logged in to the domain.
	 * @throws RemoteException 
	 */
	public Problem getNextProblem(String userName, String domainName)
	throws DomainNotFoundException, DomainNotLoadedException, InvalidProblemException,
	UserNotFoundException, UserNotLoggedInException, RemoteException;
	
	/**
	 * Set the custom problem selection algorithm for a domain. A class implementing
	 * ProblemSelectionAlgorithm should be in a jar located in
	 * domains/<domain-name>/CustomProblemSelectionAlgorithm.jar
	 * This will be different if you are not using the default path
	 * @param domainName The domain name
	 * @param fileName The class name of the class in the jar
	 * @throws DomainNotFoundException If the domain doesn't exist
	 * @throws DomainNotLoadedException If the domain is not loaded
	 */
	public void setCustomProblemSelectionAlgorithm(String domainName, String fileName) 
	throws DomainNotFoundException, DomainNotLoadedException;
	
	/**
	 * Set the custom feedback object for a domain. A class implementing
	 * Feedback should be in a jar located in
	 * domains/<domain-name>/Feedback.jar
	 * This will be different if you are not using the default path
	 * @param domainName The domain namee
	 * @param fileName The class name of the class in the jar
	 * @throws DomainNotFoundException If the domain doesn't exist
	 * @throws DomainNotLoadedException If the domain is not loaded
	 */
	public void setCustomFeedback(String domainName, String fileName) 
	throws DomainNotFoundException, DomainNotLoadedException;
	
	/**
	 * Gets the next problem, specified by the system. The system runs an
	 * algorithm to determine what problem to give the student next.
	 * The problem is returned as a Problem Object
	 * @param userName The name of the user who wants the problem.
	 * @param domainName The name of the domain.
	 * @return The problem object
	 * @throws DomainNotFoundException If the domain cannot be found.
	 * @throws DomainNotLoadedException If the requested domain is not loaded.
	 * @throws InstantiationException If instantiation fails.
	 * @throws IllegalAccessException when an application tries to reflectively
	 * create an instance (other than an array),set or get a field, or invoke a method.
	 * @throws RemoteException 
	 * @throws UserNotFoundException 
	 * @throws InvalidProblemException 
	 */
	public Problem getProblem(String userName, String domainName)
	throws DomainNotFoundException, DomainNotLoadedException, InstantiationException,
	IllegalAccessException, RemoteException, UserNotFoundException, InvalidProblemException;	
	
	/**
	 * Gets all the problems in the current working subdomain of the domain 
	 * and returns them as a SortedMap. This can then be used by the users to choose what problem
	 * they want to do next.
	 * @param userName The name of the user who wants the problem.
	 * @param domainName The name of the domain.
	 * @return A sorted map of Problem objects
	 * @throws DomainNotFoundException If the domain cannot be found.
	 * @throws DomainNotLoadedException If the requested domain is not loaded.
	 * @throws UserNotFoundException If the user could not be found.
	 * @throws UserNotLoggedInException If the user is not logged in to the domain.
	 * @throws RemoteException 
	 */
	public SortedMap<Integer, Problem> getAllProblems(String userName,String domainName)
	throws DomainNotFoundException, DomainNotLoadedException, UserNotFoundException,
	UserNotLoggedInException, RemoteException;
	
	/**
	 * Get the status of a problem for a particular domain and user.
	 * @param userName The name of the user
	 * @param domainName The name of the domain
	 * @param problemID The id of the problem you want the status of
	 * @return The status of the problem
	 * @throws UserNotFoundException If the user could not be found.
	 * @throws UserNotLoggedInException If the user is not logged in to the domain.
	 * @throws DomainNotFoundException If the domain cannot be found.
	 * @throws DomainNotLoadedException If the requested domain is not loaded.
	 * @throws RemoteException
	 */
	public String getProblemStatus(String userName, String domainName, int problemID) 
	throws UserNotFoundException, UserNotLoggedInException, DomainNotFoundException, 
	DomainNotLoadedException, RemoteException;
	
	/**
	 * Used when a student wishes to submit their solution to the current
	 * problem. This method will evaluate the student's solution against
	 * the domain constraints, and returns a Feedback Object. 
	 * The constraints and attempt both need to be sent to the evaluator,
	 * which will do the actual evaluation. A Solution has the following structure:
	 * (("CASE" "N")("GENDER" "N")("ARTICLE" "D")("ANSWER" "graue"))
	 * This method uses the Students current problem to the the ideal solution
	 * @param studentSolution the students attempt, in a String of key-value pairs.
	 * @param userName The name of the user who wants the problem.
	 * @param domainName The name of the domain.
	 * @return A Feedback Object.
	 * @throws DomainNotFoundException If the supplied domain cannot be found.
	 * @throws DomainNotLoadedException If the requested domain is not loaded.
	 * @throws InvalidSolutionException If either the students or ideal solution is invalid.
	 * @throws StudentModelNotFoundException If the students StudentModel cannot be found.
	 * @throws UserNotFoundException If the user could not be found.
	 * @throws UserNotLoggedInException If the user is not logged in to the domain.
	 * @throws JessException If something goes wrong in Jess (the rule engine)
	 * @throws RemoteException
	 */
	public Feedback submitSolution(Solution studentSolution, String userName, String domainName)
	throws DomainNotFoundException, DomainNotLoadedException, InvalidSolutionException,
	StudentModelNotFoundException, UserNotFoundException, UserNotLoggedInException,
	JessException, RemoteException;
}
