package jitas.xml.writer;

import jitas.util.Logger;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import com.sun.org.apache.xml.internal.serialize.OutputFormat;
import com.sun.org.apache.xml.internal.serialize.XMLSerializer;

/**
 * The abstract class provides the basic functions for users to write the
 * content to XML file.
 * 
 * @author	DeathMarch (c314g2)
 * @version	22 August 2007
 */
public abstract class XMLWriter {
	
	/**
	 * Load the XML file with given path name.
	 * If the pathName doesn't exist, we will attempt to create it, and
	 * we will return an empty Document. If our creation of a file 
	 * was unsuccessful, we print an error to the log, and 
	 * return a null Document. 
	 * @param pathName
	 * @return parsedDoc
	 */
	public Document loadXML(String pathName){
		DocumentBuilder db = null;
		Document parsedDoc = null; 
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        
        /*Create the document builder*/
        try {
            db = dbf.newDocumentBuilder();
        } catch (ParserConfigurationException pce) {
            System.err.println(pce);
            System.exit(1);
        }
        db.setErrorHandler(new DefaultHandler());
        
        /* If there's some weird reason we can't write, return a null document */
        if(! checkWeCanWriteToFile(pathName)) return null;
        
        /*If the file is empty, return an empty new Document */
        if(isFileEmpty(pathName)) return db.newDocument();
        
        
        /*Load the xml doc, log any exceptions*/
        try {
            parsedDoc = db.parse(pathName);
        } catch (SAXException se) {
        	Logger.addEntry("We have a SAX Exception in XMLWriter.loadXML" +
        				    ", Error message is: \n\n" + se.toString() + "\n\n\n");
        } catch (IOException ioe) {
        	Logger.addEntry("We have an IOException in XMLWriter.loadXML" +
				    ", Error message is: \n\n" + ioe.toString() + "\n\n\n");
        	return null;
        }
        return parsedDoc;
	}
	
	/**
	 * Write the content to XML file with given document and path name
	 * If something goes wrong, and the file can't write, we don't write anything.
	 * @param parsedDoc
	 */
	public void writeToXML(Document parsedDoc, String pathName){
		try{
			if(!checkWeCanWriteToFile(pathName))return;
			OutputFormat format = new OutputFormat(parsedDoc);
			format.setIndenting(true);

			/* To generate a file output use fileoutputstream instead of system.out */
			XMLSerializer serializer = new XMLSerializer(
			new FileOutputStream(new File(pathName)), format);
			serializer.serialize(parsedDoc);
		} catch(IOException ie) {
			ie.printStackTrace();
		}	
	}
	
	/*
	 * Returns true if the file exists, if the file didn't exist, the method
	 * will attempt to create it, and if that fails, then this method will return 
	 * false.
	 */
	private boolean checkWeCanWriteToFile(String pathName) {
		 /*See if the file actually exists! if not, create it.*/
        File file = new File(pathName);
        if(!file.exists()) {
        	Logger.addEntry("XMLWriter had to create " + pathName + " as this did not exist!\n");
        	try {
				file.createNewFile();
			} catch (IOException e) {
				Logger.addEntry("IOException was caught in XMLWriter!!! aborting file creation.\n\n " +
						"printStackTrace: \n" + e.toString() +"\n\n\n");
				return false;
			}
        }
        return true;
	}
	
	
	/*
	 * tests to see whether a file is empty or not
	 * param pathName the location of the file
	 * return true if the file is empty, else false.
	 */
	private boolean isFileEmpty(String pathName) {
		File f = new File(pathName);
    	try {
			FileReader reader = new FileReader(f);
			if(reader.read() == -1) return true;
		} catch (FileNotFoundException e) {
		} catch (IOException e) {}
		
		return false;
	}
}
